import React, { Component } from 'react'
import PropTypes from 'prop-types'
import { css } from 'glamor'
import Pane from './Pane'
import Overlay from './Overlay'
import Text from './Text'
import Button from './Button'
import theme from './theme'
const { majorScale, minorScale } = theme

const BottomSheetTopBar = ({ left, right, children, ...props }) =>
  <Pane display='flex' top='0' position='fixed' width='100%' style={{ zIndex: '10' }} alignItems='center' shadow white {...props}>
    <Pane width='33%' height='100%' padding={minorScale(2)} display='flex'>{left}</Pane>
    <Pane flexAuto textAlign='center'>{children}</Pane>
    <Pane width='33%' textAlign='right' padding={minorScale(2)}>{right}</Pane>
  </Pane>

BottomSheetTopBar.propTypes = {
  left: PropTypes.node,
  right: PropTypes.node
}

BottomSheetTopBar.displayName = 'BottomSheetTopBar'

const BottomSheetCloseButton = (props) => <Button paddingX={majorScale(3)} {...props}>Cancel</Button>

const slideInAnimation = css.keyframes('slideInAnimation', {
  from: {
    transform: `translateY(100%)`
  },
  to: {
    transform: `translateY(0)`
  }
})

const slideOutAnimation = css.keyframes('slideOutAnimation', {
  from: {
    transform: `translateY(0)`
  },
  to: {
    transform: `translateY(100%)`
  }
})

const BottomSheetPane = ({ slideOut, ...props }) => <Pane {...css({
  transform: `translateZ(0)`,
  animation: `${slideOut ? slideOutAnimation : slideInAnimation} ${theme.animationDuration.slow} ${theme.animationEasing.deceleration}`
})} {...props} />

BottomSheetPane.propTypes = {
  slideOut: PropTypes.bool
}

BottomSheetPane.defaultProps = {
  tint: true,
  position: 'absolute',
  shadow: true,
  height: '100vh',
  paddingTop: majorScale(7),
  left: 0,
  right: 0,
  bottom: 0
}

BottomSheetPane.displayName = 'BottomSheetPane'

class BottomSheet extends Component {
  constructor (props, context) {
    super(props, context)

    this.handleClose = this.handleClose.bind(this)
    this.handleAnimationEnd = this.handleAnimationEnd.bind(this)

    this.state = {
      isClosing: false
    }
  }

  handleClose () {
    this.setState({ isClosing: true })
  }

  handleAnimationEnd () {
    if (!this.state.isClosing) return
    this.props.onClose()
    this.setState({ isClosing: false })
  }

  render () {
    const {
      isOpen,
      children,
      navigationBarLeft,
      navigationBarRight,
      title,
      ...props
    } = this.props

    const { isClosing } = this.state

    if (!isOpen) return null

    return (
      <Overlay onClick={this.handleClose} fadeOut={isClosing} onFadeEnd={this.handleAnimationEnd}>
        <BottomSheetPane onClick={(e) => e.stopPropagation()} slideOut={isClosing} {...props}>
          <BottomSheetTopBar left={React.cloneElement(navigationBarLeft, { onClick: this.handleClose })} right={navigationBarRight}>
            <Text is='h2' textStyle='display4'>{title}</Text>
          </BottomSheetTopBar>
          <Pane style={{ overflowY: 'auto' }} height='100%'>
            {children}
          </Pane>
        </BottomSheetPane>
      </Overlay>
    )
  }
}

BottomSheet.propTypes = {
  isOpen: PropTypes.bool,
  onClose: PropTypes.func,
  navigationBarLeft: PropTypes.node,
  navigationBarRight: PropTypes.node,
  title: PropTypes.string
}

BottomSheet.defaultProps = {
  isOpen: true,
  navigationBarLeft: <BottomSheetCloseButton />,
  navigationBarRight: null,
  title: null
}

BottomSheet.displayName = 'BottomSheet'

export default BottomSheet
