import React from 'react'
import PropTypes from 'prop-types'
import { css } from 'glamor'
import cx from 'classnames'
import Box from './Box'
import theme from './theme'
const { majorScale, colors, rawColors } = theme

const intentStateStyle = intentState => ({
  backgroundColor: intentState.backgroundColor,
  color: intentState.textColor,
  ...(intentState.border ? {
    border: intentState.border
  } : {}),
  ...(intentState.boxShadow ? {
    boxShadow: intentState.boxShadow
  } : {})
})

const intentStyle = intent => ({
  ...intentStateStyle(intent.default),
  '&:hover': {
    ...intentStateStyle(intent.hover)
  },
  '&:focus': {
    ...intentStateStyle(intent.focus)
  },
  [`&:active, &.${theme.activeClassName}`]: {
    ...intentStateStyle(intent.active)
  },
  [`&:disabled, &.${theme.activeClassName}:disabled`]: {
    cursor: 'default',
    backgroundColor: colors.backgroundTint2,
    color: rawColors.grey300,
    border: 'none'
  }
})

const getStyle = ({ primary, secondary, danger }) => {
  if (primary) return css(intentStyle(theme.intent['primary']))
  if (danger) return css(intentStyle(theme.intent['danger']))
  if (secondary) return css(intentStyle(theme.intent['secondary'])) /* default */
  return ''
}

const baseStyles = css({
  textTransform: 'capitalize',
  border: 0,
  fontWeight: 400,
  textDecoration: 'none',
  transition: theme.animationDuration.faster,
  transitionProperty: 'background-color, color, box-shadow, opacity',
  cursor: 'pointer',
  whiteSpace: 'nowrap',
  userSelect: 'none',
  verticalAlign: 'middle',
  '&:active, &:focus': {
    outline: 'none'
  }
})

function useControlSizes ({ size, ...props }) {
  const sizes = theme.controlSizes[size]
  const paddingX = {
    small: majorScale(2),
    default: majorScale(3)
  }

  return {
    minHeight: sizes.minHeight,
    minWidth: sizes.minWidth,
    textStyle: sizes.textStyle,
    paddingX: paddingX[size],
    ...props
  }
}

const parseProps = ({
  className,
  primary,
  secondary,
  danger,
  ...props
}) => ({
  className: cx(`${baseStyles}`, `${getStyle({ primary, secondary, danger })}`, className),
  ...props
})

const Button = props => <Box {...parseProps(useControlSizes(props))} />

Button.propTypes = {
  ...Box.propTypes,
  primary: PropTypes.bool,
  secondary: PropTypes.bool,
  danger: PropTypes.bool,
  size: PropTypes.oneOf(Object.keys(theme.controlSizes))
}

Button.defaultProps = {
  is: 'button',
  display: 'inline-flex',
  alignItems: 'center',
  justifyContent: 'center',
  size: 'default',
  secondary: true,
  rounded: true
}

Button.displayName = 'Button'

export default Button
