import React, { Component } from 'react'
import PropTypes from 'prop-types'
import TransitionGroup from 'react-transition-group/TransitionGroup'
import { css } from 'glamor'
import theme from './theme'
import cx from 'classnames'
import objectValues from 'object-values'
const {
  majorScale,
  animationDuration,
  animationEasing,
  colors,
  borderRadius,
  popoverShadow
} = theme
import Box from './Box'
import Portal from './Portal'

const ANIMATION_LEAVE_DURATION = parseInt(animationDuration.slow, 10)

const styles = {}

styles.position = {
  position: 'absolute',
  zIndex: 50,
  transition: `all ${animationDuration.slow} ${animationEasing.deceleration}`,
  '&.popover-hide': {
    opacity: '0'
  }
}

styles.animation = {
  transition: `all ${animationDuration.fast} ${animationEasing.spring}`,
  transform: 'scale(0) translateY(-20px)',
  '&[data-position="bottom"]': {
    opacity: 1,
    visibility: 'visible',
    transform: 'scale(1) translateY(4px)'
  },
  '&[data-position="top"]': {
    opacity: 1,
    visibility: 'visible',
    transform: 'scale(1) translateY(-4px)'
  }
}

styles.arrow = {
  position: 'absolute',
  zIndex: 1,
  '&[data-position="bottom"]': {
    transform: 'translateY(-10px) translateX(-50%)'
  },
  '&[data-position="top"]': {
    bottom: 0,
    transform: 'translateY(10px) translateX(-50%)  rotate(180deg)'
  }
}

styles.content = {
  overflow: 'hidden',
  position: 'relative',
  minWidth: majorScale(20),
  border: `1px solid ${colors.borderColor}`,
  borderRadius: borderRadius[0],
  boxShadow: popoverShadow,
  background: 'white'
}

const PopoverSides = {
  TOP: 'top',
  BOTTOM: 'bottom'
}

class PopoverBox extends Component {
  constructor () {
    super()
    this.state = {
    }
  }

  render () {
    const { children, onRef, useArrow } = this.props
    const { top, left, arrowLeft, side } = this.state

    const transformOrigin = `${arrowLeft}px ${side === PopoverSides.BOTTOM ? '-20px' : 'bottom'}`

    const arrowStyle = {
      ...(useArrow ? {
        left: arrowLeft
      } : {
        display: 'none'
      })
    }

    return (
      <div
        {...css(styles.position)}
        style={{ left, top }}
        ref={(node) => { this.position = node; onRef(node) }}
        data-position={side}
        data-alignment='center'>

        <div style={{ transformOrigin }} {...css(styles.animation)} ref={(node) => { this.animation = node }}>
          <svg data-position={side} style={arrowStyle} {...css(styles.arrow)} ref={(node) => { this.arrow = node }} width='26' height='11' viewBox='0 0 26 11' xmlns='http://www.w3.org/2000/svg'>
            <path d='M21.698 8.11c.936 1.015 2.922 1.887 4.297 1.887h-26c1.378 0 3.357-.868 4.297-1.888L10.91.92c1.14-1.237 3.034-1.233 4.17 0l6.618 7.185z' fill='#E4EDF5' />
            <path d='M20.964 8.787c1.125 1.22 3.376 2.21 5.03 2.21H28-2 .005c1.654 0 3.9-.983 5.03-2.21l6.616-7.183c.75-.81 1.96-.802 2.7 0l6.62 7.183z' fill='#FFF' />
          </svg>

          <div {...css(styles.content)}>
            {children}
          </div>
        </div>
      </div>
    )
  }

  componentWillEnter (callback) {
    const { useSmartPositioning } = this.props
    const { top, bottom } = this.props.anchors
    let side = this.props.side

    // Smartly position the popover when it overflows the body
    const popoverRect = this.position.getBoundingClientRect()
    const bodyWidth = document.body.clientWidth
    const bodyHeight = document.body.clientHeight

    if (useSmartPositioning && popoverRect.height + bottom.y > bodyHeight) {
      side = 'top'
    }

    this.animation.setAttribute('data-position', side)
    const OFFSET = 4

    let left = Math.max(bottom.x - popoverRect.width / 2, OFFSET)
    left = Math.min(left, bodyWidth - popoverRect.width - OFFSET)

    let arrowLeft
    if (popoverRect.width / 2 - OFFSET < bottom.x) {
      arrowLeft = popoverRect.width / 2 - OFFSET
      arrowLeft = Math.max(arrowLeft, bottom.x - left)
    } else {
      arrowLeft = bottom.x
      arrowLeft = Math.max(arrowLeft - OFFSET, bottom.x - left)
    }

    this.setState({
      left: left,
      side,
      top: side === PopoverSides.BOTTOM ? bottom.y : top.y - popoverRect.height,
      arrowLeft
    })

    setTimeout(callback, 17)
  }

  componentDidEnter () {
    this.position.classList.add('popover-show')
    this.position.classList.remove('popover-hide')
  }

  componentWillLeave (callback) {
    this.position.classList.remove('popover-show')
    this.position.classList.add('popover-hide')
    setTimeout(() => {
      callback()
      this.props.onAnimationEnd()
    }, ANIMATION_LEAVE_DURATION)
  }
}

PopoverBox.propTypes = {
  children: PropTypes.node.isRequired,
  onRef: PropTypes.func.isRequired,
  side: PropTypes.oneOf(objectValues(PopoverSides)),
  anchors: PropTypes.shape({
    top: PropTypes.shape({
      x: PropTypes.number,
      y: PropTypes.number
    }),
    bottom: PropTypes.shape({
      x: PropTypes.number,
      y: PropTypes.number
    })
  }),
  useArrow: PropTypes.bool,
  useSmartPositioning: PropTypes.bool
}

PopoverBox.defaultProps = {
  onRef: () => {},
  onAnimationEnd: () => {}
}

PopoverBox.displayName = 'PopoverBox'

class Popover extends Component {
  constructor () {
    super()

    this.state = {
      isOpen: false,
      anchors: {
        top: {
          x: 0,
          y: 0
        },
        bottom: {
          x: 0,
          y: 0
        }
      }
    }

    this.onButtonClick = this.onButtonClick.bind(this)
    this.onBodyClick = this.onBodyClick.bind(this)
    this.onResize = this.onResize.bind(this)
    this.handleRef = this.handleRef.bind(this)
    this.onEsc = this.onEsc.bind(this)
  }

  render () {
    const {
      onOpen, // eslint-disable-line
      onClose, // eslint-disable-line
      children,
      content,
      openClass,
      side,
      useSmartPositioning,
      useArrow,
      ...props
    } = this.props
    const { isOpen, anchors } = this.state

    return (
      <Box {...props} onRef={(node) => { this.wrapper = node }}>
        {React.cloneElement(children, {
          className: cx(children.props.className, isOpen && openClass),
          onClick: this.onButtonClick
        })}
        <Portal>
          <TransitionGroup>
            {isOpen && <PopoverBox
              onRef={this.handleRef}
              anchors={anchors}
              side={side}
              useSmartPositioning={useSmartPositioning}
              useArrow={useArrow}
              >{content}</PopoverBox>}
          </TransitionGroup>
        </Portal>
      </Box>
    )
  }

  componentWillUnmount () {
    document.body.removeEventListener('click', this.onBodyClick, false)
    document.body.removeEventListener('keydown', this.onEsc, false)
    window.removeEventListener('resize', this.resize, false)
  }

  handleRef (node) {
    this.popoverNode = node
  }

  getAnchors () {
    const targetRect = this.wrapper.childNodes[0].getBoundingClientRect()
    const bodyRect = document.body.getBoundingClientRect()
    const x = targetRect.left + (targetRect.width / 2)
    return {
      top: {
        x,
        y: targetRect.top - bodyRect.top
      },
      bottom: {
        x,
        y: targetRect.bottom - bodyRect.top
      }
    }
  }

  onButtonClick (e) {
    const isOpen = !this.state.isOpen

    if (isOpen) {
      this.open()
    } else {
      this.close()
    }

    this.setState({ isOpen })
  }

  onBodyClick (e) {
    // Ignore clicks on the popover or button
    if (this.wrapper === e.target || this.wrapper.contains(e.target)) {
      return
    }

    if (this.popoverNode && (this.popoverNode === e.target || this.popoverNode.contains(e.target))) {
      return
    }

    this.close()
  }

  onEsc (e) {
    // esc key
    if (e.keyCode === 27) {
      this.close()
    }
  }

  onResize () {
    this.close()
  }

  open () {
    if (this.state.isOpen) {
      return
    }

    this.setState({ isOpen: true, anchors: this.getAnchors() })
    document.body.addEventListener('click', this.onBodyClick, false)
    document.body.addEventListener('keydown', this.onEsc, false)
    window.addEventListener('resize', this.onResize, false)

    this.props.onOpen()
  }

  close () {
    if (!this.state.isOpen) {
      return
    }

    this.setState({ isOpen: false })
    document.body.removeEventListener('click', this.onBodyClick, false)
    document.body.removeEventListener('keydown', this.onEsc, false)
    window.removeEventListener('resize', this.resize, false)

    this.props.onClose()
  }
}

Popover.propTypes = {
  ...Box.propTypes,
  children: PropTypes.element.isRequired,
  content: PropTypes.node.isRequired,
  openClass: PropTypes.string,
  onOpen: PropTypes.func.isRequired,
  onClose: PropTypes.func.isRequired,
  side: PropTypes.oneOf(objectValues(PopoverSides)),
  useArrow: PropTypes.bool,
  useSmartPositioning: PropTypes.bool
}

Popover.defaultProps = {
  side: PopoverSides.BOTTOM,
  openClass: theme.activeClassName,
  onOpen: () => {},
  onClose: () => {},
  useArrow: false,
  useSmartPositioning: true
}

Popover.displayName = 'Popover'

export default Popover
