import React, {Component} from 'react'
import PropTypes from 'prop-types'
import {css} from 'glamor'
import Box from './Box'
import theme from './theme'
import offscreenCss from './utils/offscreenCss'
const {majorScale, borderRadius, controlStateStyles} = theme
let count = 1 // Used for generating unique input names

const labelClass = css({
  display: 'flex',
  flex: 1,
  alignItems: 'center',
  justifyContent: 'center',
  position: 'relative',
  ...controlStateStyles.default
})

const wrapperClass = css({
  position: 'relative',
  display: 'flex',
  flex: 1,
  cursor: 'pointer',
  marginLeft: '-1px',
  [`:first-child .${labelClass}`]: {
    borderTopLeftRadius: borderRadius[0],
    borderBottomLeftRadius: borderRadius[0]
  },
  [`:last-child .${labelClass}`]: {
    borderTopRightRadius: borderRadius[0],
    borderBottomRightRadius: borderRadius[0]
  }
})

const inputClass = css({
  ':hover, :focus, :active': {
    [`& + .${labelClass}`]: {
      zIndex: 1
    }
  },
  [`:hover + .${labelClass}`]: {
    ...controlStateStyles.hover
  },
  [`:active + .${labelClass}`]: {
    ...controlStateStyles.active
  },
  [`:checked + .${labelClass}`]: {
    ...controlStateStyles.checked
  }
})

function useControlSizes (size) {
  const sizes = theme.controlSizes[size]

  return {
    minHeight: sizes.minHeight,
    minWidth: sizes.minWidth,
    textStyle: sizes.textStyle,
    paddingX: majorScale(1)
  }
}

function SegmentedControlRadio (props) {
  const {size, name, label, value, checked, onChange} = props

  return (
    <label className={`${wrapperClass}`}>
      <input
        type='radio'
        className={`${offscreenCss} ${inputClass}`}
        name={name}
        value={value}
        checked={checked}
        onChange={(e) => onChange(e.target.value)} />

      <Box className={`${labelClass}`} {...useControlSizes(size)}>
        {label}
      </Box>
    </label>
  )
}

SegmentedControlRadio.propTypes = {
  size: PropTypes.oneOf(Object.keys(theme.controlSizes)),
  name: PropTypes.string.isRequired,
  label: PropTypes.node.isRequired,
  value: PropTypes.string.isRequired,
  checked: PropTypes.bool.isRequired,
  onChange: PropTypes.func.isRequired
}

SegmentedControlRadio.displayName = 'SegmentedControlRadio'

const parseProps = ({
  className,
  options,
  name,
  value,
  defaultValue,
  onChange,
  ...props
}) => ({
  ...props
})

class SegmentedControl extends Component {
  constructor (props) {
    super()
    const {options, defaultValue} = props

    this.state = {
      value: defaultValue || options[0].value
    }

    // Default name if the name prop isn't set
    this.name = `SegmentedControl-${count}`
    count += 1

    this.onChange = this.onChange.bind(this)
  }

  render () {
    const {options, size} = this.props
    const name = this.props.name || this.name
    // Allows it to behave like a controlled input
    const value = this.props.value || this.state.value

    return (
      <Box display='flex' marginRight='-1px' {...parseProps(this.props)}>
        {options.map((option) => (
          <SegmentedControlRadio
            key={option.value}
            size={size}
            name={name}
            label={option.label}
            value={option.value}
            checked={value === option.value}
            onChange={this.onChange} />
        ))}
      </Box>
    )
  }

  onChange (value) {
    // Save a render cycle when it's a controlled input
    if (!this.props.value) {
      this.setState({value})
    }

    if (this.props.onChange) {
      this.props.onChange(value)
    }
  }
}

SegmentedControl.propTypes = {
  ...Box.propTypes,
  options: PropTypes.arrayOf(PropTypes.shape({
    label: PropTypes.node.isRequired,
    value: PropTypes.string.isRequired
  })).isRequired,
  size: PropTypes.oneOf(Object.keys(theme.controlSizes)),
  name: PropTypes.string,
  value: PropTypes.string,
  defaultValue: PropTypes.string,
  onChange: PropTypes.func
}

SegmentedControl.defaultProps = {
  size: 'default'
}

SegmentedControl.displayName = 'SegmentedControl'

export default SegmentedControl
