import React from 'react'
import { storiesOf } from '@kadira/storybook'
import {
  App,
  Pane,
  Text,
  Button,
  theme,
  CodeBlock
} from '../components'
import Content from './Content'
const { minorScale, majorScale } = theme

export default function gettingStartedStory (module) {
  storiesOf('Getting Started', module)
    .add('Welcome to the React UI Library', () => (
      <App>
        <Pane flexAuto white padding={majorScale(10)} clearfix>
          <Content markdown>
            {`
# Welcome to the React UI Library for segment
> Presentational React Components with a coherent API for developers

The React UI Library is a set of presentational UI components build in React based on Segment styles.

* Uses the Segment Design System
* Powerful component API
* Dedicated UI engineering workflow
* Documentation with React Storybook
* Powerful prototyping tool

## Installing the library in your project

The library is a single dependency with all of the UI. Install with yarn or npm.

\`\`\`
$ yarn add @segment/react-ui-library
\`\`\`

Using \`npm\`:

\`\`\`
$ npm install --save @segment/react-ui-library
\`\`\`

## Using your first component

You can import all of the components straight out of the UI Library.
Here is a full working example (assuming you have a ES6, React environment).

`}
          </Content>
          <CodeBlock language='jsx'>
            {`import React from 'react'
import ReactDOM from 'react-dom'
import {
  Button
} from '@segment/react-ui-library'

const MyButton = () => <Button primary>I am using the UI Library!</Button>

ReactDOM.render(<MyButton />, document.getElementById('root'))
`}
          </CodeBlock>
          <Pane tint padding={majorScale(2)} marginTop={majorScale(2)} border>
            <Button primary>I am using the UI Library!</Button>
          </Pane>
          <Content markdown>
            {`

## Documentation

React Storybook is used for documentation (the thing you are currently reading).
The \`usage\` tab for each component is used to describe how the component is used, and potentially the properties.

Tabs under a component other than usage will contain examples of the component being used.
If you are curious to know how that is build look inside of \`react-ui-library/src/stories/component-story.js\`.
You can do this in Github, or clone the React UI Library
`}
          </Content>
        </Pane>
      </App>
    )).add('Using the spacing system', () => (
      <App>
        <Pane flexAuto white padding={majorScale(10)} clearfix>
          <Content markdown>
            {`
# Using the spacing system

The React UI Library uses a major scale based on 8px — and a minor scale based on 4px.
That means that everything should be a multiple of 8px or 4px.

## Importing the spacing scales

From v1.0.4 forward you can import the spacing scales directly from the library:
`}
          </Content>
          <CodeBlock language='jsx'>
            {`import React from 'react'
import ReactDOM from 'react-dom'
import {
  Pane,
  Button,
  minorScale, // <= importing the minor scale here
  majorScale // <= importing the major scale here
} from '@segment/react-ui-library'

const UsingSpacing = () =>
  <Pane tint padding={majorScale(2)} marginTop={majorScale(2)} border>
    <Button primary marginRight={minorScale(3)}>Primary</Button>
    <Button>Secondary</Button>
  </Pane>

ReactDOM.render(<UsingSpacing />, document.getElementById('root'))
`}
          </CodeBlock>
          <Text textStyle='display4' marginTop={majorScale(2)}>Result:</Text>
          <Pane tint padding={majorScale(2)} marginTop={majorScale(2)} border>
            <Button primary marginRight={minorScale(3)}>Primary</Button>
            <Button>Secondary</Button>
          </Pane>
          <Content markdown>
            {`

## How does majorScale and minorScale work?

The best way to understand the scales is to take a look at how they are implemented:

`}
          </Content>
          <CodeBlock language='javascript'>
            {`const majorScale = x => \`$\{x * 8}px\`
const minorScale = x => \`$\{x * 4}px\`

majorScale(1) // => '8px'
majorScale(2) // => '16px'
majorScale(3) // => '24px'
majorScale(4) // => '32px'
majorScale(5) // => '40px'

minorScale(1) // => '4px'
minorScale(3) // => '12px'
minorScale(5) // => '20px'
`}
          </CodeBlock>
          <Content markdown>
            {`




## When to use majorScale vs. minorScale

Use majorScale first. If something doesn't work try the minorScale.

* minorScale is allowed for dimensions and spacing of icons
* minorScale is allowed for spacing of small text

## Upgrading from v0 to v1

Here is a mapping from the old implicit spacing system to new explicit spacing system.

`}
          </Content>
          <CodeBlock language='javascript'>
            {`const space = [
  '0',    //  0
  '4px',  // 1: minorScale(1)
  '8px',  // 2: majorScale(1)
  '12px', // 3: minorScale(2)
  '16px', // 4: majorScale(2)
  '24px', // 5: majorScale(3)
  '32px', // 6: majorScale(4)
  '40px', // 7: majorScale(5)
  '48px', // 8: majorScale(6)
  '64px', // 9: majorScale(8)
  '80px'  // 10: majorScale(10)
]
`}
          </CodeBlock>
        </Pane>
      </App>
    ))
}
